set verify off
set echo on

drop table t;

create table t
as
select a.*, 'x' extra1, 'y' extra2
  from all_objects a;

create index t_idx1 on t(owner,object_name,extra1) ;
create index t_idx2 on t(object_name,owner,extra2) ;

exec dbms_stats.gather_table_stats( user, 'T', method_opt => 'for all indexed columns' );
pause

create or replace package state_pkg
as
	type vcArray is table of varchar2(30);
	l_owners  vcArray;
	l_object_name vcArray;
end;
/
create or replace package body state_pkg
as

begin
	select distinct owner, object_name 
	  bulk collect into l_owners, l_object_name
	  from t;
end;
/
exec dbms_output.put_line( 'There are ' || state_pkg.l_owners.count || ' elements' );
pause


column cpu new_val c
column lio new_val l

select max( decode( a.name, 'CPU used by this session', b.value  ) ) cpu,
       max( decode( a.name, 'session logical reads', b.value  ) ) lio
  from v$statname a, v$mystat b
 where a.statistic# = b.statistic# 
   and a.name in ( 'CPU used by this session', 'session logical reads' );
@trace
declare
	type array is table of t%rowtype;
	l_data  array;
begin
	for i in 1 .. state_pkg.l_owners.count
	loop
		select /*+ index(t t_idx1) */ * 
		  bulk collect 
		  into l_data 
		  from t 
		 where owner = state_pkg.l_owners(i)
		   and object_name = state_pkg.l_object_name(i);
	end loop;
end;
/

select cpu, (cpu-&c)/100 cpu_diff,
       lio, to_char( lio-&l, '999,999' ) lio_diff
  from (
select max( decode( a.name, 'CPU used by this session', b.value  ) ) cpu,
       max( decode( a.name, 'session logical reads', b.value  ) ) lio
  from v$statname a, v$mystat b
 where a.statistic# = b.statistic# 
   and a.name in ( 'CPU used by this session', 'session logical reads' )
       )
/

declare
	type array is table of t%rowtype;
	l_data  array;
begin
	for i in 1 .. state_pkg.l_owners.count
	loop
		select /*+ index(t t_idx2) */ * 
		  bulk collect 
		  into l_data 
		  from t 
		 where owner = state_pkg.l_owners(i)
		   and object_name = state_pkg.l_object_name(i);
	end loop;
end;
/
select cpu, (cpu-&c)/100 cpu_diff,
       lio, to_char( lio-&l, '999,999' ) lio_diff
  from (
select max( decode( a.name, 'CPU used by this session', b.value  ) ) cpu,
       max( decode( a.name, 'session logical reads', b.value  ) ) lio
  from v$statname a, v$mystat b
 where a.statistic# = b.statistic# 
   and a.name in ( 'CPU used by this session', 'session logical reads' )
       )
/
